#![no_std]
#![no_main]
#![feature(abi_x86_interrupt)]
#![feature(associated_type_defaults)]
#![allow(internal_features)]
#![feature(ptr_internals)]
#![feature(allocator_api)]

use core::ptr::addr_of;

use alloc::vec::Vec;
use pc_keyboard::{DecodedKey, KeyCode};
use x86_64::instructions::nop;

use crate::{gdt::PRIV_TSS_STACK, scheduler::{executor::Executor, Task}, utils::jmp_to_usermode};

extern crate alloc;

#[macro_use]
mod logger;
mod interrupts;
mod gdt;
mod utils;
#[macro_use]
mod events;
mod mem;
mod scheduler;
mod std;



#[no_mangle]
extern "C" fn kmain(mb2_info_addr: usize) -> ! {
    // Init
    logger::init(log::LevelFilter::Trace).unwrap();
    gdt::init();
    interrupts::init_idt();
    utils::enable_nxe_bit();
    utils::enable_write_protect_bit();
    std::init();
    
    let active_table = mem::init(mb2_info_addr);
    
    //unsafe {
    //    jmp_to_usermode(active_table, usermode_code as *const () as usize, addr_of!(PRIV_TSS_STACK) as *const () as usize);
    //}

    //let mut executor = Executor::new();
    //executor.spawn(Task::new(usermode_code()));
    //executor.run();

    //let mut a = Vec::new();
    

    //a.push(6);
    //a.push(9);
    //a.push(4);
    //a.push(2);
    //a.push(0);
    
    //log::error!("{:?}", a); 


    
    // EVENTMAN.lock().add_listener(events::Event::Ps2KeyPress(None), |k| {
    //    let Event::Ps2KeyPress(v) = k else {panic!()};
    //
    //    log::debug!("Keypress event received! ({:?})", v.unwrap());
    //   Ok(())
    // });

    // let (level_4_page_table, _) = Cr3::read();
    // println!("Level 4 page table at: {:?}", level_4_page_table.start_address());
    log::info!("end of work");
    loop {
        if std::input::is_key_down(KeyCode::A) {
            log::info!(":3");
        }
    }
    // utils::hcf();
}


#[panic_handler]
fn panic(pi: &core::panic::PanicInfo) -> ! {
    log::error!("{pi}");
    utils::hcf();
}


async fn usermode_code() {
    log::info!("Hello from usermode!");
}
